#include "audit_pipe_regression_test_utils.h"

void init_channel_primary()
{
	key_t key;
	key = 1000;
	
	mutex = sem_open(SEM_NAME,0,0644);
	mutex_client = sem_open(SEM_CLIENT_NAME,0, 0644);
    
	if(mutex == SEM_FAILED || mutex_client == SEM_FAILED)
    	err(-1, "sem_open");
    	
    shmid = shmget(key,SHMSZ,0644);
  	if(shmid<0)
    	err(-1, "shmget");
    shm = shmat(shmid, NULL, 0);
    
} 

void write_data(char *string)
{
	char *s;
	int value;
	
	sem_wait(mutex_client);
	s = shm;
	for(;*string != '\0';string++)
		*s++ = *string;

	*s++= '\0';
	sem_post(mutex);
}

void write_end()
{
	write_data("END");
}
	
void write_string(char *data, char *field)
{
	char *string;
	
	string = malloc(512);
	if(string == NULL)
		return;
	snprintf(string, 512, "%s\\!!%s:**STRING", data, field);
	write_data(string);
	free(string);
}

void write_int(int data, char *field)
{
	char *string;
	
	string = malloc(512);
	if(string == NULL)
		return;
	snprintf(string, 512, "%d\\!!%s:**INT", data, field);
	write_data(string);
	free(string);
}

void write_char(char data, char *field)
{
	char *string;
	
	string = malloc(512);
	if(string == NULL)
		return;
	snprintf(string, 512, "%c\\!!%s:**INT", data, field);
	write_data(string);
	free(string);
}
		
void init_channel()
{
	key_t key;
	key = 1000;

	sem_unlink(SEM_NAME);
	sem_unlink(SEM_CLIENT_NAME);

	mutex = sem_open(SEM_NAME,O_CREAT | O_EXCL,0644,0);
	mutex_client = sem_open(SEM_CLIENT_NAME,O_CREAT | O_EXCL, 0644, 1);

	if(mutex == SEM_FAILED || mutex_client == SEM_FAILED)
    	err(-1, "sem_open");

	shmid = shmget(key,SHMSZ,IPC_CREAT | 0644);
  	if(shmid<0)
		err(-1, "shmget");

	shm = shmat(shmid,NULL,0);

}

void end_channel()
{
	sem_close(mutex);
	sem_unlink(SEM_NAME);
	sem_close(mutex_client);
	sem_unlink(SEM_CLIENT_NAME);
	shmctl(shmid, IPC_RMID, 0);
}

/*
 * Read a string from shared-memory channel
 */
char *read_string()
{
	char string[512];
	char *s;
	char *ptr;
	int i = 0;
	int value;

	sem_wait(mutex);

	s = shm;

	for(;*s != '\0'; s++) {
		string[i] = *s;
		i++;
	}
	string[i] = '\0';
	ptr = string;
	sem_post(mutex_client);

	return (ptr);
}

/*
 * Every string in the shared-memory channel is of the form 
 * value\\!!audit-field:**type.
 * Here we parse the audit-field part
 */
char *get_descr(char *string)
{
	char *str;

	str = string;

	if((string = strsep(&str, "\\!!")) != NULL) {
		str +=2;
		return (str);
	}
}

/*
 * Every string in the shared-memory channel is of the form 
 * value\!!audit-field:**type.
 * Here we parse the type part
 */
int parse_string(char *string)
{
	char *str;
	if(!strncmp(string, "END", 3))
		return (-1);
	str = string;

	if((string = strsep(&str, ":**")) != NULL) {
		if(!strncmp(str, "**INT", 5))
			return (INT_TYPE);
		if(!strncmp(str, "**CHAR",6))
			return (CHAR_TYPE);
		if(!strncmp(str, "**STRING",8))
			return (STRING_TYPE);
	}
	return (-1);		
}

/*
 * We extract integers from strings
 */
long get_int(char *string)
{
	long ret;
	char *val;

	if(string == NULL)
		return (-1);

	ret = strtol(string, &val, 0);

	if(!ret && errno == EINVAL)
		return (-1);
	if(*val == '\0')
		return (ret);

	return (-1);
}

/*
 * We extract char from strings
 */
char get_c(char *string)
{	
	if(strlen(string) != sizeof(char))
		return ('\0');
	else
		return (string[0]);

	return ('\0');
}
	
/*
 * Init log file for evaluation
 */
FILE *init_log(pid_t pid)
{
	FILE *f;
	char path[512];

	snprintf(path, 512, "/tmp/audit-log-%d.xml", pid);

	f = fopen(path, "w+");
		if(f == NULL)
			err(-1, "FOPEN");

	return (f);
}

void report_error_string(char *str, FILE *f)
{
	fprintf(f, "ERROR:***\n");
	fprintf(f, str);
	fprintf(f, "\n");
}
	
void report_error(tokenstr_t tok, FILE *f)
{
	
	fprintf(f, "ERROR:***\n");
	au_print_tok_xml(f, &tok, ",", 0, 0);
	fprintf(f, "\n");

}	

int check_exit(au_exit_t ex, int val)
{
	if(ex.status == val)
		return (1);
	
	return (0);
}

/*
 * Check a sockaddr_in struct
 */
int check_sock(au_socketinet32_t s , struct sockaddr_in a)
{
	if( s.family == a.sin_family && s.port == a.sin_port && 
		s.addr == a.sin_addr.s_addr)
		return (1);
	
	return (0);
}

int check_exec_arg(char *str_exec, char *arg)
{
	
	if(!strncmp(str_exec, arg, 256))
		return (1);
	
	return (0);
}	

/*		
 * Check the return value of a bsm-token
 */
int check_ret(au_ret32_t ret, int r, char *val)
{
	if(ret.ret == r)
		if(!ret.status)
			if(!strncmp(val, "success", strlen(val)))
				return (1);
		else if(!strncmp(strerror(ret.status), val, strlen(val)))
			return (1);
	return (0);
}

/*
 * Check the text field of a bsm-token
 */
int check_text(au_text_t text, char *val)
{
	if(!text.len)
		return (-1);
	
	if(!strncmp(text.text, val, text.len > strlen(val)? text.len: strlen(val)))
		return (1);
	
	return (0);
}

/*
 * Check the path field of a bsm-token
 */
int check_path(au_path_t path, char *val)
{
	if(!path.len)
		return (-1);
	
	if(!strncmp(path.path, val, path.len > strlen(val)? path.len: strlen(val)))
		return (1);
	
	return (0);
}

/*
 * Check the arg field of a bsm-token
 */
int check_arg(au_arg32_t arg, long val)
{
	if(!arg.len)
		return (-1);
	
	if(arg.val == val)
		return (1);
	
	return (0);
}


/*
 * Check the pid and privilege of a bsm-token
 */
int check_priv(au_proc32ex_t pr, pid_t pid)
{
	int auid;
	
	if(getauid(&auid) == -1)
		return (0);
		
	if(pr.pid == pid && pr.euid == geteuid() && pr.ruid == getuid() && 
	    pr.rgid == getgid() && pr.egid == getegid() && pr.auid == auid)
		return (1);
	
	return (0);
}

/*
 * Check privilege of a child with different credential from parent.
 */
int check_priv_child(au_proc32ex_t pr, struct priv p)
{
	
	if(pr.pid == p.pid && pr.euid == p.euid && pr.ruid == p.ruid && 
	    pr.rgid == p.rgid && pr.egid == p.egid && pr.auid == p.auid)
		return (1);
	
	return (0);		
}
			
/*
 * Check the pid  and privilege of an external process
 */
int check_ext_priv(au_proc32_t pr, struct priv p)
{
	if(pr.pid == p.pid && pr.euid == p.euid && pr.ruid == p.ruid && 
	    pr.rgid == p.rgid && pr.egid == p.egid && pr.auid == p.auid)
		return (1);
	
	return (0);
}

/* 
 * Check attributes a file pointed by its path.
 */
int check_attr_path(au_attr32_t attr, char *path)
{
	struct stat s;
	
	if(stat(path, &s) == -1)
		return (0);
		
	if(attr.mode == (s.st_mode & ALLPERMS) && attr.uid == s.st_uid && attr.gid == s.st_gid && 
	    attr.nid == s.st_ino && attr.dev == s.st_dev)
		return (1);
	
	return (0);
}
	
/*
 * Check attributes of fd from a bsm-token
 */
int check_fd(au_attr32_t attr, int fd)
{
	struct stat s;

	if(fstat(fd, &s) == -1)
		return (0);
		
	if(attr.mode == (s.st_mode & ALLPERMS) && attr.uid == s.st_uid && attr.gid == s.st_gid && 
	    attr.nid == s.st_ino && attr.dev == s.st_dev)
		return (1);

	return (0);
}

int setup_auditpipe(int pid)
{
	int fd;
	int value;
	struct auditpipe_ioctl_preselect_proc *entry;
	
	fd = open("/dev/auditpipe", O_RDONLY);
	if (fd < 0)
		err(-1, "/dev/auditpipe");

	entry = malloc(sizeof(struct auditpipe_ioctl_preselect_proc));
	if(entry == NULL)
		err(-1, "MALLOC");

	value = AUDITPIPE_PRESELECT_MODE_PROC;
	if (ioctl(fd, AUDITPIPE_SET_PRESELECT_MODE, &value) < 0)
		err(-1, "AUDITPIPE_SET_PRESELECT_MODE");
		
	entry->aipp_pid = pid;

	if (ioctl(fd, AUDITPIPE_SET_PRESELECT_PROC, entry) < 0)
			err(-1, "AUDITPIPE_SET_PRESELECT_EVENTS");
	
	return (fd);
}

void close_auditpipe(int fd, int fdout)
{
	close(fd);
	close(fdout);
}

void read_auditpipe(int fd, int fdout)
{
	int len, count;
	char buffer[AUDIT_BUFFER_LEN];
	struct pollfd pollfd;

	pollfd.fd = fd;
	pollfd.events = POLLIN;
	pollfd.revents = 0;
	count = poll(&pollfd, 1, 0);
	if (count < 0) {
		switch (errno) {
			case EINTR:
				return;
			default:
				err(-1, "poll");
			}
		}
		
	if (pollfd.revents & POLLIN) {
		len = read(fd, buffer, AUDIT_BUFFER_LEN);
		if (len < 0)
			err(-1, "auditpipe read");
		if (len == 0)
			return;
			
		/* Audit pipe input. */
		if(write(fdout, buffer, len) == -1)
			err(-1, "WRITE");
	}
}
	
