/*-
 * Copyright (c) 2008 Vincenzo Iozzo <snagg@FreeBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/stat.h>
#include <sys/sysctl.h>

#include <bsm/libbsm.h>
#include <bsm/audit.h>

#include <security/audit/audit_ioctl.h>

#include <netinet/in.h>

#include <err.h>
#include <fcntl.h>
#include <poll.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <semaphore.h>
#include <string.h>

#define	AUDIT_BUFFER_LEN	65536
#define AUDIT_RECORD_MAX	256

#define SHMSZ	512
#define SEM_NAME	"/audit-test"
#define SEM_CLIENT_NAME	"/client"

#define TOKEN_FLAG_ISSET(bitmask, flag)	\
	(bitmask & flag)
#define TOKEN_FLAG_SET(bitmask, flag)	\
	(bitmask |= flag)

#define	TOKEN_ARG	0x0000001
#define	TOKEN_PATH	0x0000002
#define	TOKEN_ATTR	0x0000004
#define	TOKEN_SUBJECT	0x0000008
#define	TOKEN_RETURN	0x0000010
#define	TOKEN_EXIT	0x0000020
#define	TOKEN_EXEC_ARG	0x0000040
#define TOKEN_EXEC_ENV	0x0000080
#define	TOKEN_GROUP	0x0000100
#define	TOKEN_ADDR	0x0000200
#define	TOKEN_IP	0x0000400
#define	TOKEN_IPC	0x0000800
#define	TOKEN_PORT	0x0001000
#define	TOKEN_PROCESS	0x0002000
#define	TOKEN_SOCKET	0x0004000
#define	TOKEN_TEXT	0x0008000
#define	TOKEN_DATA	0x0010000

sem_t *mutex, *mutex_client;
char *shm;
int shmid;

enum TYPES{
	INT_TYPE,
	STRING_TYPE,
	CHAR_TYPE
};

/*
 * NOT = do not validate anything
 * VALID = we need to validate stuff
 * WAITING = this is not the right syscall to parse
 */
enum VALIDATE {
	NOT,
	VALID,
	WAITING
};

struct priv
{
	int pid;
	int euid;
	int ruid;
	int rgid;
	int egid;
	int auid;
};	

void init_channel();
void init_channel_primary();
void end_channel();
char *read_string();
void write_int(int data, char *field);
void write_string(char *data, char *field);
void write_char(char data, char *field);
void write_end();
char *get_descr(char *string);
int parse_string(char *string);
long get_int(char *string);
char get_c(char *string);
FILE *init_log(pid_t pid);
void report_error(tokenstr_t tok, FILE *f);
void report_error_string(char *str, FILE *f);
int check_exit(au_exit_t ex, int val);
int check_text(au_text_t text, char *val);
int check_ret(au_ret32_t ret, int r, char *val);
int check_path(au_path_t path, char *val);
int check_arg(au_arg32_t arg, long val);
int check_priv(au_proc32ex_t pr, pid_t pid);
int check_fd(au_attr32_t attr, int fd);
int check_ext_priv(au_proc32_t pr, struct priv p);
int check_priv_child(au_proc32ex_t pr, struct priv p);
int check_attr_path(au_attr32_t attr, char *path);
int check_sock(au_socketinet32_t s , struct sockaddr_in a);
int check_exec_arg(char *str_exec, char *arg);
